package com.example.demokafka.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.example.demokafka.common.api.vo.Result;
import com.example.demokafka.dto.GuangZhaoChuanGanQiDTO;
import com.example.demokafka.dto.HuanJingJianCeDTO;
import com.example.demokafka.dto.KafkaConsumerDTO;
import com.example.demokafka.dto.KongQiZhiLiangChuanGanQiDTO;
import com.example.demokafka.util.KafkaUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description
 * @Author ZhouWenTao
 * @Date 2023/9/5 9:57
 */
@RestController
@RequestMapping("/kafka")
@Slf4j
@Api(tags = "Kafka接口")
public class KafkaController {
    @Resource
    private KafkaUtil kafkaUtil;
    @GetMapping("/consumer/{topic}")
    @ApiOperation(value = "consumer")
    public Result<?> getConsumerData(@PathVariable String topic) {
        if (StringUtils.isEmpty(topic)) {
            return Result.error("请确认topic是否正确");
        }

        List<String> resultList = new ArrayList<>();
        Consumer<String, String> consumer = new KafkaConsumer<>(kafkaUtil.getSafeConsumerPro());
        consumer.subscribe(Collections.singletonList(topic));
        ConsumerRecords<String, String> records = consumer.poll(5000);
        JSONArray jsonArray=new JSONArray();
        for (ConsumerRecord<String, String> consumerRecord : records) {
            String kfkContent = consumerRecord.value();
            resultList.add(kfkContent);
            if (!StringUtils.isEmpty(kfkContent)) {
                jsonArray.add(JSONObject.parseObject(kfkContent));
            }
        }
        consumer.close();



        long newCreateTime =0L;

        switch (topic) {
            case "guangzhaochuanganqi": {
                //光照传感器
                List<JSONObject> jsonObjects = jsonArray.toJavaList(JSONObject.class);
                List<KafkaConsumerDTO<GuangZhaoChuanGanQiDTO>> kafkaConsumerDTOList;
                KafkaConsumerDTO<GuangZhaoChuanGanQiDTO> kafkaConsumerDTO;
                //排除掉旧的重复aeid
                Map<String, KafkaConsumerDTO<GuangZhaoChuanGanQiDTO>> aeidMap = new HashMap<>();
                for (JSONObject jsonObject : jsonObjects) {
                    kafkaConsumerDTO = jsonObject.toJavaObject(KafkaConsumerDTO.class);
                    String aeid = kafkaConsumerDTO.getAeid();
                    String createtimeStr = kafkaConsumerDTO.getCreatetime();
                    newCreateTime = Long.parseLong(createtimeStr);
                    KafkaConsumerDTO<GuangZhaoChuanGanQiDTO> kafkaConsumerDTO1 = aeidMap.get(kafkaConsumerDTO.getAeid());
                    if (kafkaConsumerDTO1 == null) {
                        aeidMap.put(aeid, kafkaConsumerDTO);
                    } else {
                        long createtime = Long.parseLong(kafkaConsumerDTO1.getCreatetime() == null ? "0" : kafkaConsumerDTO1.getCreatetime());
                        if (StringUtils.isEmpty(kafkaConsumerDTO.getCreatetime())) {
                            continue;
                        }
                        if (createtime >= newCreateTime) {
                            continue;
                        }
                    }
                    aeidMap.put(aeid, kafkaConsumerDTO);
                }
                kafkaConsumerDTOList = new ArrayList<>(aeidMap.values());
                return Result.OK(kafkaConsumerDTOList);
            }
            case "kongqizhiliangchuanganqi": {
                //空气质量传感器
                List<JSONObject> jsonObjects = jsonArray.toJavaList(JSONObject.class);
                List<KafkaConsumerDTO<KongQiZhiLiangChuanGanQiDTO>> kafkaConsumerDTOList;
                KafkaConsumerDTO<KongQiZhiLiangChuanGanQiDTO> kafkaConsumerDTO;
                //排除掉旧的重复aeid
                Map<String, KafkaConsumerDTO<KongQiZhiLiangChuanGanQiDTO>> aeidMap = new HashMap<>();
                for (JSONObject jsonObject : jsonObjects) {
                    kafkaConsumerDTO = jsonObject.toJavaObject(KafkaConsumerDTO.class);
                    String aeid = kafkaConsumerDTO.getAeid();
                    String createtimeStr = kafkaConsumerDTO.getCreatetime();
                    newCreateTime = Long.parseLong(createtimeStr);
                    KafkaConsumerDTO<KongQiZhiLiangChuanGanQiDTO> kafkaConsumerDTO1 = aeidMap.get(kafkaConsumerDTO.getAeid());
                    if (kafkaConsumerDTO1 == null) {
                        aeidMap.put(aeid, kafkaConsumerDTO);
                    } else {
                        long createtime = Long.parseLong(kafkaConsumerDTO1.getCreatetime() == null ? "0" : kafkaConsumerDTO1.getCreatetime());
                        if (StringUtils.isEmpty(kafkaConsumerDTO.getCreatetime())) {
                            continue;
                        }
                        if (createtime >= newCreateTime) {
                            continue;
                        }
                    }
                    aeidMap.put(aeid, kafkaConsumerDTO);
                }
                kafkaConsumerDTOList = new ArrayList<>(aeidMap.values());
                return Result.OK(kafkaConsumerDTOList);
            }
            case "huanjingjiance": {
                //环境监测
                List<JSONObject> jsonObjects = jsonArray.toJavaList(JSONObject.class);
                List<KafkaConsumerDTO<HuanJingJianCeDTO>> kafkaConsumerDTOList = new ArrayList<>();
                KafkaConsumerDTO<HuanJingJianCeDTO> kafkaConsumerDTO;
                //排除掉旧的重复aeid
                Map<String, KafkaConsumerDTO<HuanJingJianCeDTO>> aeidMap = new HashMap<>();
                for (JSONObject jsonObject : jsonObjects) {
                    kafkaConsumerDTO = jsonObject.toJavaObject(KafkaConsumerDTO.class);
                    String aeid = kafkaConsumerDTO.getAeid();
                    String createtimeStr = kafkaConsumerDTO.getCreatetime();
                    newCreateTime = Long.parseLong(createtimeStr);
                    KafkaConsumerDTO<HuanJingJianCeDTO> kafkaConsumerDTO1 = aeidMap.get(kafkaConsumerDTO.getAeid());
                    if (kafkaConsumerDTO1 == null) {
                        aeidMap.put(aeid, kafkaConsumerDTO);
                    } else {
                        long createtime = Long.parseLong(kafkaConsumerDTO1.getCreatetime() == null ? "0" : kafkaConsumerDTO1.getCreatetime());
                        if (StringUtils.isEmpty(kafkaConsumerDTO.getCreatetime())) {
                            continue;
                        }
                        if (createtime >= newCreateTime) {
                            continue;
                        }
                    }
                    aeidMap.put(aeid, kafkaConsumerDTO);
                }
                kafkaConsumerDTOList = new ArrayList<>(aeidMap.values());
                return Result.OK(kafkaConsumerDTOList);
            }
            default:
                return Result.error("请确认topic是否正确");
        }
    }

}
